const EXPORTED_SYMBOLS = ['TabDockUtils', 'EventSupport', 'SetAttributeCounter'];

/**
 * A dreaded utils class, which contains odds and ends. Since we're not supposed
 * to use prototypes of built in types for extensions, prototype-y things go
 * here as well.
 */
var TabDockUtils = (function() {
	
	var debugEnabled = true;
	
	return {
		
		/**
         * Sets the debug flag, used to determine whether or not to dump info.
         * 
         * @param enable
         *            Enables dump statements if true, disables otherwise.
         */
		enableDebug: function(enable) {
			debugEnabled = enable ? true : false;
		},
	
		/**
         * Dumps a message, if debug is enabled. TODO - find out if there is
         * some sort of string formatter function that can be used.
         * 
         * @param messageString
         *            The messageString to dump out.
         */
		dump: function(messageString) {
			if(debugEnabled)
				dump(messageString);
		},
		
		/**
         * Dumps an object, if debug is enabled.
         * 
         * @param object
         *            The object to dump out.
         */
		dumpObject: function(object) {
			if(debugEnabled) {
				var output = '';
				for (property in object) {
					output += property + ': ' + object[property] + '; \n';
				}

				dump(output);
			}
		},

		/**
         * Removes all children from the given node.
         * 
         * @param node
         *            The node to remove from
         */
		removeAllChildren: function(node) {
			while (node.childNodes.length >= 1)
				node.removeChild(node.firstChild);
		},
		
		/**
         * Goes through all the children in a node and sets the ordinal value.
         * The boxObject is used to traverse all the child nodes in ordinal
         * value order.
         * 
         * The node ordinals applied are spaced out with one number between
         * adjacent groups. This allows for easy reordering by setting the
         * ordinal of an object to one off from a target sibling, then calling
         * reapplyOrdinals to allow the process to be repeated.
         * 
         * @param container
         *            A node, containing sub-nodes, to space the ordinals out
         *            for.
         */
		reapplyOrdinals: function(container) {
			let orderedObj = Array();
			let currentObj = container.boxObject.firstChild;
			
			while(currentObj != null) {
				orderedObj.push(currentObj);
				currentObj = currentObj.boxObject.nextSibling;
			}
			
			for(let i = 0; i < orderedObj.length; i++)
				orderedObj[i].ordinal = (i*2) + 2;
		},
		
		/**
         * Removes the first instance of an object in the array
         * 
         * @param obj
         *            The object to remove from the array
         */
		removeFromArray: function(array,  obj) {
			for( var i = 0; i < array.length; i++)
				if (array[i] === obj)
					return array.splice(i, 1);
		},
		
        /**
         * Finds the index for a given object.
         * 
         * @param obj
         *            The object to locate.
         */
		getIndexInArray: function(array, obj) {
	          for( var i = 0; i < array.length; i++)
	                if (array[i] === obj)
	                    return i;
	          
	          throw new Error("Object not found");
		},
		
		
		/**
         * Generates a tab preview for the given tab, based on a 16:9 aspect
         * ratio of the screen size (not application window size).
         * 
         * @param aTab
         *            The tab to generate a preview for
         * @return A canvas element containing the tab preview
         */
	    generatePreview: function(aTab) {
	            let win = aTab.linkedBrowser.contentWindow;
	            
	            let aspectRatio = 0.5625; // 16:9
	            let screen = win.screen;
	            
	            let width = Math.ceil(screen.availWidth / 5.75);
	            let height = Math.round(width * aspectRatio);
	                      
	            return TabDockUtils.drawWindow(aTab, width, height);
	    },
	    
        /**
         * Tab previews utility, produces thumbnails. Code based on browser.js
         * tabPreviews. Unlike the Firefox function, it uses the full window of
         * the tab to generate a preview rather than just a upper left portion.
         * 
         * @param aTab
         *            The tab to generate a preview for
         * @param width
         *            The width of the preview
         * @param height
         *            The height of the preview
         * @return A canvas element containing the tab preview
         */
        drawWindow: function(aTab, width, height) {
            let win = aTab.linkedBrowser.contentWindow;
            
            let aspectRatio = 0.5625; // 16:9
            let screen = win.screen;
                      
            let canvas = win.document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
            canvas.mozOpaque = true;
            canvas.height = height;
            canvas.width = width;

            let ctx = canvas.getContext("2d");
            /*
             * FIXME - window's innerWidth doesn't update instantly, which can
             * cause this to mess up if done too quickly after the window is
             * resized. This results in a black bar appearing on the right or
             * bottom of the preview.
             */
            let snippetWidth = win.innerWidth * 1.0;
            let snippetHeight = win.innerHeight * 1.0;
            let scaleWidth = width / snippetWidth;
            let scaleheight = height / snippetHeight;
            
            ctx.scale(scaleWidth, scaleheight);
            ctx.drawWindow(win, win.scrollX, win.scrollY,
                    snippetWidth, snippetHeight, "rgb(255,255,255)");

            return canvas;
        }
	}
})();

/**
 * A very basic event support for firing off events to listeners. Listeners can
 * be either added as 'functions' or objects. If they are added as objects, then
 * the handleEvent function is called, bound to the object itself.
 */
function EventSupport() {
	this._typeSpecificListeners = {};
	this._allTypeListeners = [];
	
	this._fireForListeners = function(event, listeners) {
        for (let i=0; i < listeners.length; i++) {
        	try {
        		if (typeof(listeners[i]) == "function") {
        	        listeners[i].call(this, event);
        		} else {
        			/*
                     * TODO - just bind once when added, that is
                     * _listeners[type].push =
                     * listener.handleEvent.bind(listener).
                     */
        			listeners[i].handleEvent.bind(listeners[i]).call(this, event);
        		}
        	} catch(err) {
        		// Make sure all other listeners still get to go
        		TabDockUtils.dump("error while firing listener: " + err + "\n");
                TabDockUtils.dump(err.stack);
        	}
        }
	};
	
	this._getListeners = function(type) {
	    if(type)
	        return this._typeSpecificListeners[type] = this._typeSpecificListeners[type] || [];
	    else
	        return this._allTypeListeners;
	};
};

EventSupport.prototype = {

    addListener: function(listener, type){
        this._getListeners(type).push(listener);
    },

    fire: function(event, type){
    	event.type = type;

    	this._fireForListeners(event, this._getListeners(type));
        this._fireForListeners(event, this._getListeners());
    },

    removeListener: function(listener, type){
        TabDockUtils.removeFromArray(this._getListeners(type), listener);
    },
};



/**
 * A counter that can count up or down. If a specified threshold value is
 * reached (either going up or down), then an attribute (with a value) is
 * applied to an object.
 * 
 * @param minValue
 *            The minimum value this counter can be set to.
 * @param initialValue
 *            What value the counter should start at.
 */
function SetAttributeCounter(minValue, initialValue) {
	
	var self = this;
	
	self.Threshold = function(obj, attr, val) {
		this.obj = obj;
		this.attr = attr;
		this.val = val;
	}
	
	this.minValue = minValue;
	this.currentValue = initialValue || minValue;

	this.thresholds = {};
};

SetAttributeCounter.prototype.increment = function() {
	
	this.currentValue++;
	var thresholdActions = this.thresholds[this.currentValue];
	if(thresholdActions !== undefined)
		for(let i=0; i<thresholdActions.length; i++) {
		    let threshold = thresholdActions[i];
			threshold && threshold.obj.setAttribute(threshold.attr, threshold.val);
		}
	
	return this.currentValue;
};

SetAttributeCounter.prototype.decrement = function() {
	if(this.currentValue == this.minValue)
		return;
	
	this.currentValue--;
	let thresholdActions = this.thresholds[this.currentValue];
	if(thresholdActions !== undefined)
		for(let i=0; i<thresholdActions.length; i++) {
		    let threshold = thresholdActions[i];
			threshold && threshold.obj.setAttribute(threshold.attr, threshold.val);
		}


	return this.currentValue;
};

SetAttributeCounter.prototype.value = function() {
	return this.currentValue;
};

SetAttributeCounter.prototype.addThreshold = function(value, object, attribute, attributeValue) {
	
	if(this.thresholds[value] === undefined)
		this.thresholds[value] = new Array();
	
	this.thresholds[value].push(new this.Threshold(object, attribute, attributeValue));
	
	if(value == this.currentValue)
		object.setAttribute(attribute, attributeValue);
};
