const EXPORTED_SYMBOLS = ['TabEvents'];

Components.utils.import('chrome://tabdock/content/tabdock-utils.js');

/**
 * Wrapper around browser tab events.
 */
var TabEvents = function(aWindow) {
    
    this._window = aWindow;
    
    /** For firing tab events */
    this._eventSupport = new EventSupport();
    
    /** Keeps track of the previous title for each tab */
    this._tabTitleMap = new WeakMap();
    
    /** The tabs that have a title changed event */
    this._tabTitleChangedMap = new WeakMap();
    
    /** All the tabs that are unread (really a set) */
    this._tabUnreadMap = new WeakMap();
    
    /** The tabs that are busy */
    this._tabBusyMap = new WeakMap();
    
    /** Number of tabs that have a title change event */
    this.titleChangedTabsCount = 0;
    
    /** Number of tabs that have an unread event */
    this.unreadTabsCount = 0;
    
    /** Number of tabs that have a busy event */
    this.busyTabsCount = 0;
    
    aWindow.TabEvents = this;
    aWindow.TabEvents.setup();
}

// Event types
TabEvents.EVENT_TAB_OPENED = 'EVENT_TAB_OPENED';
TabEvents.EVENT_TAB_CLOSED = 'EVENT_TAB_CLOSED';
TabEvents.EVENT_TAB_SELECTED = 'EVENT_TAB_SELECTED';
TabEvents.EVENT_TAB_ATTR_CHANGED = 'EVENT_TAB_ATTR_CHANGED';
TabEvents.EVENT_TAB_TITLE_UPDATED = 'EVENT_TAB_TITLE_UPDATED';
TabEvents.EVENT_TAB_TITLE_CHANGED = 'EVENT_TAB_TITLE_CHANGED';
TabEvents.EVENT_TAB_TITLE_CHANGED_CLEARED = 'EVENT_TAB_TITLE_CHANGED_CLEARED';
TabEvents.EVENT_TAB_UNREAD = 'EVENT_TAB_UNREAD';
TabEvents.EVENT_TAB_UNREAD_CLEARED = 'EVENT_TAB_UNREAD_CLEARED';
TabEvents.EVENT_TAB_BUSY = 'EVENT_TAB_BUSY';
TabEvents.EVENT_TAB_BUSY_CLEARED = 'EVENT_TAB_BUSY_CLEARED';

TabEvents.prototype = {
    
    isTitleChanged: function(aTab) {
        return this._tabTitleChangedMap.has(aTab);
    },
    
    isUnread: function(aTab) {
        return this._tabUnreadMap.has(aTab);
    },
    
    setTitleChange: function(aTab) {
        
        let tabStringBundle = this._window.document.getElementById("tab-strings");
        let connectingString = tabStringBundle.getString("tabs.connecting");
        
        let oldLabel = this._tabTitleMap.get(aTab);
        
        if(!aTab.selected
            && aTab.label != oldLabel
            && !this._tabTitleChangedMap.has(aTab)
            && aTab.label != connectingString
            && oldLabel != connectingString
            && oldLabel != undefined)
        {
            this.titleChangedTabsCount++;
        
            this._tabTitleChangedMap.set(aTab, "");
            this._eventSupport.fire( { 'tab' : aTab }, TabEvents.EVENT_TAB_TITLE_CHANGED);
        }
        
    },
    
    clearTitleChange: function(aTab) {
        
        if(this._tabTitleChangedMap.has(aTab)) {
            this.titleChangedTabsCount--;
            
            this._tabTitleChangedMap.delete(aTab);
            this._eventSupport.fire( { 'tab' : aTab }, TabEvents.EVENT_TAB_TITLE_CHANGED_CLEARED);
        }
        
    },
    
    setUnread: function(aTab) {
        
        if(!aTab.selected && !this._tabUnreadMap.has(aTab)) {
            this.unreadTabsCount++;
            
            this._tabUnreadMap.set(aTab, aTab.label);
            this._eventSupport.fire( { 'tab' : aTab }, TabEvents.EVENT_TAB_UNREAD);
        }
        
    },
    
    clearUnread: function(aTab) {
        
        if(this._tabUnreadMap.has(aTab)) {
            this.unreadTabsCount--;
            
            this._tabUnreadMap.delete(aTab);
            this._eventSupport.fire( { 'tab' : aTab }, TabEvents.EVENT_TAB_UNREAD_CLEARED);
        }
        
    },
    
    setBusy: function(aTab) {
        
        if(aTab.hasAttribute("busy") && !this._tabBusyMap.has(aTab)) {
            this.busyTabsCount++;
            
            this._tabBusyMap.set(aTab, aTab.label);
            this._eventSupport.fire( { 'tab' : aTab }, TabEvents.EVENT_TAB_BUSY);
        }
        
    },
    
    clearBusy: function(aTab) {
        
        if(!aTab.hasAttribute("busy") && this._tabBusyMap.has(aTab)) {
            this.busyTabsCount--;
            
            this._tabBusyMap.delete(aTab);
            this._eventSupport.fire( { 'tab' : aTab }, TabEvents.EVENT_TAB_BUSY_CLEARED);
        }
        
    },

    onTabAttrChange: function(aTab) {
    	// ignore events on closed tabs
    	if(aTab.closed)
    		return;
      
        let oldLabel = this._tabTitleMap.get(aTab);
        
        if(aTab.label != oldLabel) {
            this._eventSupport.fire( { 'tab' : aTab }, TabEvents.EVENT_TAB_TITLE_UPDATED);
        }
        
        this.setBusy(aTab);
        this.clearBusy(aTab);
        this.setTitleChange(aTab);
        this._tabTitleMap.set(aTab, aTab.label); 
        
        this._eventSupport.fire( { 'tab' : aTab }, TabEvents.EVENT_TAB_ATTR_CHANGED);
    },
    
    onTabOpen: function(aTab) {
        this._tabTitleMap.set(aTab, aTab.label);
        this._eventSupport.fire( { 'tab' : aTab }, TabEvents.EVENT_TAB_OPENED);
        
        this.setUnread(aTab);
    },
    
    onTabSelect: function(aTab)  {
    	// ignore events on closed tabs
    	if(aTab.closed)
    		return;
    
        this.clearUnread(aTab);
        this.clearTitleChange(aTab);
        
        this._eventSupport.fire( { 'tab' : aTab }, TabEvents.EVENT_TAB_SELECTED);
    },
    
    onTabClose: function(aTab) {
    	aTab.closed = true;
    
        this.clearBusy(aTab);
        this.clearUnread(aTab);
        this.clearTitleChange(aTab);
        
        this._eventSupport.fire( { 'tab' : aTab }, TabEvents.EVENT_TAB_CLOSED);
    },
    
    handleEvent: function(aEvent) {

        switch (aEvent.type)
        {
            case "TabOpen":
                return this.onTabOpen(aEvent.target);
            case "TabSelect":
                return this.onTabSelect(aEvent.target);
            case "TabClose":
                return this.onTabClose(aEvent.target);
            case "TabAttrModified":
                return this.onTabAttrChange(aEvent.target);
        }
    },
    
    setup: function() {
        let container = this._window.gBrowser.tabContainer;
        
        container.addEventListener("TabOpen", this, false);  
        container.addEventListener("TabSelect", this, false);
        container.addEventListener("TabClose", this, false);
        container.addEventListener("TabAttrModified", this, false);
    },
    
    cleanup: function() {
        let container = this._window.gBrowser.tabContainer;
        
        container.removeEventListener("TabOpen", this, false);  
        container.removeEventListener("TabSelect", this, false);  
        container.removeEventListener("TabClose", this, false);
        container.removeEventListener("TabAttrModified", this, false);
        
        delete this._window.TabEvents;
    },
    
    addListener: function(listener, type) { this._eventSupport.addListener(listener, type); },
    removeListener: function(listener, type) { this._eventSupport.removeListener(listener, type); },
}