const EXPORTED_SYMBOLS = ['TabPreviews'];

Components.utils.import('chrome://tabdock/content/tabdock-utils.js');
Components.utils.import('chrome://tabdock/content/tabdock-settings.js');
Components.utils.import('chrome://tabdock/content/preferences/preferences.js');


var TabPreviews = function(aWindow) {
    this.window = aWindow;
    this.tabpreviewsPanel = this.window.document.getElementById('tabpreviewsPanel');
    this.taboverlays = this.window.document.getElementById('tabdock-overlay');
    
    /* Maps tabs to tabpreviewitems */
    this.tabPreviewMap = null;
    
    /* Number of tabs to show on the screen at once */
    this.previewsToDisplay = 0;
    
    /* The width of each preview canvas */
    this.previewWidth = 0;
    
    /* The height of each preview canvas */
    this.previewHeight = 0;
    
    /* How much to scale the preview (after previewWidth/previewHeight) */
    this.scaleFactor = .98;
    
    /* The index of the currently selected tab */
    this.currentIndex = 0;
    
    /* The thickness of the border */
    this.borderSize = 4;
    
    /* The amount of space between each preview item */
    this.previewSpacing = 10;
    
    /* The amount of vertical space taken by the tab label */
    this.tabLabelHeight = 25;
    
    /* The preview corresponding to the current (or last) selected tab */
    this.lastSelectedPreview = null;
    
    /* Whether or not the previews are currently open */
    this.currentlyOpen = false;
    
    /* Tab group that is currently open */
    this.tabgroup = '';
    
    /* Whether or not the previews are showing */
    this.showing = false;
    
}

TabPreviews.prototype = {
    
    /**
     * Sets up a preview by generating the tab preview image, setting the label
     * and the unread / title changed states. Can be used to update the image /
     * label for an existing preview.
     */
    setupPreview: function(aTab) {
        let previewContainer = this.tabPreviewMap.get(aTab);
        
        if(!previewContainer) {
            previewContainer = this.window.document.createElement('xstabpreviewItem');
            previewContainer.style.MozBinding = "url(chrome://tabdock/content/tabpreviews/tabpreviewItem.xml#tabpreviewItem)";
            
            this.tabPreviewMap.set(aTab, previewContainer); 
            this.tabpreviewsPanel.appendChild(previewContainer);
        }
        
        let preview = TabDockUtils.drawWindow(aTab, this.previewWidth, this.previewHeight);
        previewContainer.setup(preview, aTab.label);
        
        previewContainer.unread = this.window.TabEvents.isUnread(aTab);
        previewContainer.titlechanged = this.window.TabEvents.isTitleChanged(aTab);
    },
    

    open: function(aTabGroup) {
        if(aTabGroup.tabCount == 0)
            return;
        
        if(this.closePreviewTimerId)
            this.window.clearTimeout(this.closePreviewTimerId);
        
        TabDockUtils.removeAllChildren(this.tabpreviewsPanel);
        
        this.tabgroup = aTabGroup;
        this.currentIndex = 0;
        this.tabPreviewMap = new WeakMap();
        
        this.previewsToDisplay = CurrentPrefs['previews.tabCount'];

        this.setupDimensions();

        for(let tab in aTabGroup)
            this.setupPreview(tab);
 
        this.show();
        this.tabpreviewsPanel.setAttribute('previewsOpen', 'true');
        this.taboverlays.setAttribute('previewsOpen', 'true');
        this.currentlyOpen = true;
        this.setSelected(this.window.gBrowser.selectedTab, aTabGroup);
    },
    
    isActive: function() {
        return this.currentlyOpen && this.showing; 
    },
    
    setupDimensions: function() {
        let cellPadding = Math.ceil((this.previewsToDisplay * this.previewSpacing) / (this.previewsToDisplay + 1));
        let previewPadding = (2 * this.borderSize) + cellPadding;
        
        if(CurrentPrefs['location'] == "left" || CurrentPrefs['location'] == "right")
            previewPadding += this.tabLabelHeight;

        let appcontent = this.window.document.getElementById('appcontent');
        this.previewWidth = Math.floor(this.scaleFactor * appcontent.boxObject.width / this.previewsToDisplay) - previewPadding
        this.previewHeight = Math.floor(this.scaleFactor * appcontent.boxObject.height / this.previewsToDisplay) - previewPadding;

        this.tabpreviewsPanel.maxHeight = (this.previewHeight + previewPadding) * this.previewsToDisplay;
        this.tabpreviewsPanel.maxWidth = (this.previewWidth + previewPadding) * this.previewsToDisplay;
    },

    close: function(aTabGroup) {
        if(this.tabgroup != aTabGroup)
            return;
        
        if(this.closePreviewTimerId)
            this.window.clearTimeout(this.closePreviewTimerId);
        
        if(this.showPreviewTimerId)
            this.window.clearTimeout(this.showPreviewTimerId);
        
        /*
         * Use a slight delay to prevent flickering when switching from one
         * group to another.
         */
        let closeFunction = function() {
            TabDockUtils.removeAllChildren(this.tabpreviewsPanel);
            this.tabpreviewsPanel.setAttribute('previewsOpen', 'false');
            this.taboverlays.setAttribute('previewsOpen', 'false');
            this.currentlyOpen = false;
        }.bind(this);
        
        this.closePreviewTimerId = this.window.setTimeout(closeFunction, TabPreviews.COLLAPSE_TIME_DELAY);
    },
    
    /**
     * This is used to show the previews after a popup menu is closed. There
     * doesn't appear to be a way to tell from mouseenter/mouseover what the
     * actual element we are hovering over is. Whenever the menu is open, it
     * appears to be the menu element and not the tabdock-item. It seems
     * impossible to know if we are still hovering over the tabdock.
     * 
     * Also. the onpopuphidden also appears to fire before the mouseexit event
     * fires, since the mouse is still hovering over the menu at that point.
     * This means that we don't know if we are hovering over the icon or not.
     * 
     * The workaround is to delay showing, so that the mouseexit has time to
     * fire and the previews can be closed.
     */
    show: function() {
        if(this.showPreviewTimerId)
            this.window.clearTimeout(this.showPreviewTimerId);
        
        let showFunction = function() {
            this.showing = true;
            this.tabpreviewsPanel.setAttribute('show', 'true');
            this.taboverlays.setAttribute('previewsShowing', 'true');
        }.bind(this);
        
        this.showPreviewTimerId = this.window.setTimeout(showFunction, TabPreviews.SHOW_TIME_DELAY);
    },
    
    hide: function() {   
        if(this.showPreviewTimerId)
            this.window.clearTimeout(this.showPreviewTimerId);
      
        this.showing = false;
        this.tabpreviewsPanel.setAttribute('show', 'false');
        this.taboverlays.setAttribute('previewsShowing', 'false');
    },
    
    /**
     * Selects a tab in the preview, centering the preview pane around that tab
     * (if odd) or close to the center otherwise. If there are not enough tabs
     * on the right or left of the selected tab, then the preview pane is not
     * scrolled.
     * 
     * @param aTab
     *            The tab to set as selected *
     */
    setSelected: function(aTab, aTabGroup) {
        if(!this.currentlyOpen || (aTabGroup != this.tabgroup))
            return;
        
        let preview = this.tabPreviewMap.get(aTab);
        if(!preview)
            return;
        
        let previews = this.tabpreviewsPanel.childNodes;
        let newIndex = TabDockUtils.getIndexInArray(previews, preview);
        let offset = newIndex - this.currentIndex;
        
        if(this.lastSelectedPreview)
            this.lastSelectedPreview.selected = false;
        
        preview.selected = true;
        preview.titlechanged = false;
        preview.unread = false;
        
        let scrollStartCount = Math.floor(this.previewsToDisplay / 2);
        let scrollEndCount = Math.ceil(this.previewsToDisplay / 2);
   
        /*
         * If at the start and scrolling to the current location, make sure we
         * don't scroll all the way to the item, because the ones prior to it
         * will not show.
         */
        if(this.currentIndex == 0)
            offset -=  Math.floor(this.previewsToDisplay / 2);         
        
        // Don't scroll around edges
        if((offset > 0 && newIndex > scrollStartCount) || (offset < 0 && newIndex < previews.length - scrollEndCount))
            this.tabpreviewsPanel.scrollByIndex(offset);

        this.currentIndex = newIndex;
        this.lastSelectedPreview = preview;


    },
    
    removeTab: function(aTab, aTabGroup) {
        if(!this.currentlyOpen || (aTabGroup != this.tabgroup))
            return;
        
        let preview = this.tabPreviewMap.get(aTab);
        if(preview && preview.parentNode === this.tabpreviewsPanel)
            this.tabpreviewsPanel.removeChild(preview);
        
        if(aTabGroup.tabCount == 0)
            this.hide();
    },
    
    addTab: function(aTab, aTabGroup) {
        if(!this.currentlyOpen || (aTabGroup != this.tabgroup))
            return;
        
        this.setupPreview(aTab);
    },
    
    setTitleChanged: function(aTab, aTabGroup) {
        if(!this.currentlyOpen || (aTabGroup !== this.tabgroup))
            return;
        
        this.tabPreviewMap.get(aTab).titlechanged = true;
     },
    
    /**
     * Updates the preview for a given tab, changing the image as well as the
     * title. TODO - make sure not to update too often?
     * 
     * @param aTab
     *            The tab to update the preview for
     * @param aTabGroup
     *            The tabgroup that the tab belongs to
     */
    updatePreview: function(aTab, aTabGroup) {
        if(!this.currentlyOpen || (aTabGroup !== this.tabgroup))
            return;
        
        if(this.tabPreviewMap.has(aTab))
            this.setupPreview(aTab);
        else
            this.open(aTabGroup);
    },
}

/** Amount of time to wait before collapsing the tabpreviews panel */
TabPreviews.COLLAPSE_TIME_DELAY = 100;

/** How long to delay before showing the tab previews */
TabPreviews.SHOW_TIME_DELAY = 5;